@@ -38,6 +38,7 @@
 #include "cogl-internal.h"
 #include "cogl-boxed-value.h"
 #include "cogl-pipeline-snippet-private.h"
+#include "cogl-pipeline-state.h"
 
 #include <glib.h>
 
@@ -187,10 +188,11 @@ typedef enum
   /* non-sparse */
   COGL_PIPELINE_STATE_REAL_BLEND_ENABLE_INDEX,
 
-  COGL_PIPELINE_STATE_COUNT,
-  COGL_PIPELINE_STATE_SPARSE_COUNT = COGL_PIPELINE_STATE_COUNT - 1,
+  COGL_PIPELINE_STATE_COUNT
 } CoglPipelineStateIndex;
 
+#define COGL_PIPELINE_STATE_SPARSE_COUNT (COGL_PIPELINE_STATE_COUNT - 1)
+
 /* Used in pipeline->differences masks and for notifying pipeline
  * state changes.
  *
@@ -665,9 +667,18 @@ _cogl_pipeline_prune_redundant_ancestry (CoglPipeline *pipeline);
 void _cogl_pipeline_update_blend_enable (CoglPipeline *pipeline,
                                          CoglPipelineState changes);
 
+typedef enum
+{
+  COGL_PIPELINE_GET_LAYER_NO_CREATE
+} CoglPipelineGetLayerFlags;
+
 CoglPipelineLayer *
-_cogl_pipeline_get_layer (CoglPipeline *pipeline,
-                          int layer_index);
+_cogl_pipeline_get_layer_with_flags (CoglPipeline *pipeline,
+                                     int layer_index,
+                                     CoglPipelineGetLayerFlags flags);
+
+#define _cogl_pipeline_get_layer(p, l) \
+  _cogl_pipeline_get_layer_with_flags (p, l, 0)
 
 gboolean
 _cogl_is_pipeline_layer (void *object);
@@ -960,10 +971,6 @@ _cogl_pipeline_find_equivalent_parent (CoglPipeline *pipeline,
                                        CoglPipelineState pipeline_state,
                                        CoglPipelineLayerState layer_state);
 
-CoglTexture *
-_cogl_pipeline_get_layer_texture (CoglPipeline *pipeline,
-                                  int layer_index);
-
 void
 _cogl_pipeline_get_layer_combine_constant (CoglPipeline *pipeline,
                                            int layer_index,
