@@ -35,16 +35,9 @@
 
 G_BEGIN_DECLS
 
-#define COGL_PIXEL_BUFFER(array)     ((CoglPixelBuffer *)(array))
-
 struct _CoglPixelBuffer
 {
   CoglBuffer            _parent;
-
-  CoglPixelFormat       format;
-  unsigned int          width;
-  unsigned int          height;
-  unsigned int          stride;
 };
 
 G_END_DECLS
