@@ -49,9 +49,9 @@
  * extensions match then it will be used.
  */
 
-/* The function names in OpenGL 2.0 are different so we can't easily
-   just check for GL 2.0 */
-COGL_EXT_BEGIN (shaders_glsl, 2, 0,
+/* This lists functions that are unique to GL 2.0 or GLES 2.0 and are
+ * not in the old GLSL extensions */
+COGL_EXT_BEGIN (shaders_glsl_2_only, 2, 0,
                 COGL_EXT_IN_GLES2,
                 "\0",
                 "\0")
@@ -59,25 +59,13 @@ COGL_EXT_FUNCTION (GLuint, glCreateProgram,
                    (void))
 COGL_EXT_FUNCTION (GLuint, glCreateShader,
                    (GLenum                shaderType))
-COGL_EXT_FUNCTION (void, glShaderSource,
-                   (GLuint                shader,
-                    GLsizei               count,
-                    const char          **string,
-                    const GLint          *length))
-COGL_EXT_FUNCTION (void, glCompileShader,
-                   (GLuint                shader))
 COGL_EXT_FUNCTION (void, glDeleteShader,
                    (GLuint                shader))
 COGL_EXT_FUNCTION (void, glAttachShader,
                    (GLuint                program,
                     GLuint                shader))
-COGL_EXT_FUNCTION (void, glLinkProgram,
-                   (GLuint                program))
 COGL_EXT_FUNCTION (void, glUseProgram,
                    (GLuint                program))
-COGL_EXT_FUNCTION (GLint, glGetUniformLocation,
-                   (GLuint                program,
-                    const char           *name))
 COGL_EXT_FUNCTION (void, glDeleteProgram,
                    (GLuint                program))
 COGL_EXT_FUNCTION (void, glGetShaderInfoLog,
@@ -85,23 +73,50 @@ COGL_EXT_FUNCTION (void, glGetShaderInfoLog,
                     GLsizei               maxLength,
                     GLsizei              *length,
                     char                 *infoLog))
+COGL_EXT_FUNCTION (void, glGetProgramInfoLog,
+                   (GLuint                program,
+                    GLsizei               bufSize,
+                    GLsizei              *length,
+                    char                 *infoLog))
 COGL_EXT_FUNCTION (void, glGetShaderiv,
                    (GLuint                shader,
                     GLenum                pname,
                     GLint                *params))
+COGL_EXT_FUNCTION (void, glGetProgramiv,
+                   (GLuint                program,
+                    GLenum                pname,
+                    GLint                *params))
+COGL_EXT_FUNCTION (void, glDetachShader,
+                   (GLuint program, GLuint shader))
+COGL_EXT_FUNCTION (void, glGetAttachedShaders,
+                   (GLuint program,
+                    GLsizei maxcount,
+                    GLsizei* count,
+                    GLuint* shaders))
+COGL_EXT_FUNCTION (GLboolean, glIsShader,
+                   (GLuint shader))
+COGL_EXT_FUNCTION (GLboolean, glIsProgram,
+                   (GLuint program))
+COGL_EXT_END ()
 
-COGL_EXT_FUNCTION (void, glVertexAttribPointer,
-                   (GLuint		 index,
-                    GLint		 size,
-                    GLenum		 type,
-                    GLboolean		 normalized,
-                    GLsizei		 stride,
-                    const GLvoid		*pointer))
-COGL_EXT_FUNCTION (void, glEnableVertexAttribArray,
-                   (GLuint		 index))
-COGL_EXT_FUNCTION (void, glDisableVertexAttribArray,
-                   (GLuint		 index))
-
+/* These functions are provided by GL_ARB_shader_objects or are in GL
+ * 2.0 core */
+COGL_EXT_BEGIN (shader_objects_or_gl2, 2, 0,
+                COGL_EXT_IN_GLES2,
+                "ARB\0",
+                "shader_objects\0")
+COGL_EXT_FUNCTION (void, glShaderSource,
+                   (GLuint                shader,
+                    GLsizei               count,
+                    const char          **string,
+                    const GLint          *length))
+COGL_EXT_FUNCTION (void, glCompileShader,
+                   (GLuint                shader))
+COGL_EXT_FUNCTION (void, glLinkProgram,
+                   (GLuint                program))
+COGL_EXT_FUNCTION (GLint, glGetUniformLocation,
+                   (GLuint                program,
+                    const char           *name))
 COGL_EXT_FUNCTION (void, glUniform1f,
                    (GLint                 location,
                     GLfloat               v0))
@@ -194,18 +209,39 @@ COGL_EXT_FUNCTION (void, glGetUniformiv,
                    (GLuint                program,
                     GLint                 location,
                     GLint                *params))
+COGL_EXT_FUNCTION (void, glGetActiveUniform,
+                   (GLuint program,
+                    GLuint index,
+                    GLsizei bufsize,
+                    GLsizei* length,
+                    GLint* size,
+                    GLenum* type,
+                    GLchar* name))
+COGL_EXT_FUNCTION (void, glGetShaderSource,
+                   (GLuint shader,
+                    GLsizei bufsize,
+                    GLsizei* length,
+                    GLchar* source))
+COGL_EXT_FUNCTION (void, glValidateProgram, (GLuint program))
+COGL_EXT_END ()
 
-COGL_EXT_FUNCTION (void, glGetProgramiv,
-                   (GLuint                program,
-                    GLenum                pname,
-                    GLint                *params))
-
-COGL_EXT_FUNCTION (void, glGetProgramInfoLog,
-                   (GLuint                program,
-                    GLsizei               bufSize,
-                    GLsizei              *length,
-                    char                 *infoLog))
-
+/* These functions are provided by GL_ARB_vertex_shader or are in GL
+ * 2.0 core */
+COGL_EXT_BEGIN (vertex_shaders, 2, 0,
+                COGL_EXT_IN_GLES2,
+                "ARB\0",
+                "vertex_shader\0")
+COGL_EXT_FUNCTION (void, glVertexAttribPointer,
+                   (GLuint		 index,
+                    GLint		 size,
+                    GLenum		 type,
+                    GLboolean		 normalized,
+                    GLsizei		 stride,
+                    const GLvoid        *pointer))
+COGL_EXT_FUNCTION (void, glEnableVertexAttribArray,
+                   (GLuint		 index))
+COGL_EXT_FUNCTION (void, glDisableVertexAttribArray,
+                   (GLuint		 index))
 COGL_EXT_FUNCTION (void, glVertexAttrib1f, (GLuint indx, GLfloat x))
 COGL_EXT_FUNCTION (void, glVertexAttrib1fv,
                    (GLuint indx, const GLfloat* values))
@@ -220,17 +256,14 @@ COGL_EXT_FUNCTION (void, glVertexAttrib4f,
                    (GLuint index, GLfloat x, GLfloat y, GLfloat z, GLfloat w))
 COGL_EXT_FUNCTION (void, glVertexAttrib4fv,
                    (GLuint indx, const GLfloat* values))
-
 COGL_EXT_FUNCTION (void, glGetVertexAttribfv,
                    (GLuint index, GLenum pname, GLfloat* params))
 COGL_EXT_FUNCTION (void, glGetVertexAttribiv,
                    (GLuint index, GLenum pname, GLint* params))
 COGL_EXT_FUNCTION (void, glGetVertexAttribPointerv,
                    (GLuint index, GLenum pname, GLvoid** pointer))
-
 COGL_EXT_FUNCTION (GLint, glGetAttribLocation,
                    (GLuint program, const char *name))
-
 COGL_EXT_FUNCTION (void, glBindAttribLocation,
                    (GLuint program,
                     GLuint index,
@@ -243,32 +276,4 @@ COGL_EXT_FUNCTION (void, glGetActiveAttrib,
                     GLint* size,
                     GLenum* type,
                     GLchar* name))
-COGL_EXT_FUNCTION (void, glGetActiveUniform,
-                   (GLuint program,
-                    GLuint index,
-                    GLsizei bufsize,
-                    GLsizei* length,
-                    GLint* size,
-                    GLenum* type,
-                    GLchar* name))
-COGL_EXT_FUNCTION (void, glDetachShader,
-                   (GLuint program, GLuint shader))
-COGL_EXT_FUNCTION (void, glGetAttachedShaders,
-                   (GLuint program,
-                    GLsizei maxcount,
-                    GLsizei* count,
-                    GLuint* shaders))
-COGL_EXT_FUNCTION (void, glGetShaderSource,
-                   (GLuint shader,
-                    GLsizei bufsize,
-                    GLsizei* length,
-                    GLchar* source))
-
-COGL_EXT_FUNCTION (GLboolean, glIsShader,
-                   (GLuint shader))
-COGL_EXT_FUNCTION (GLboolean, glIsProgram,
-                   (GLuint program))
-
-COGL_EXT_FUNCTION (void, glValidateProgram, (GLuint program))
-
 COGL_EXT_END ()
