@@ -151,8 +151,6 @@ main (int argc, char **argv)
   CoglDepthState depth_state;
   gboolean has_swap_notify;
 
-  g_type_init ();
-
   ctx = cogl_context_new (NULL, &error);
   if (!ctx) {
       fprintf (stderr, "Failed to create context: %s\n", error->message);
@@ -223,9 +221,9 @@ main (int argc, char **argv)
   data.texture = cogl_texture_new_from_file (COGL_EXAMPLES_DATA "crate.jpg",
                                              COGL_TEXTURE_NO_SLICING,
                                              COGL_PIXEL_FORMAT_ANY,
-                                             NULL);
+                                             &error);
   if (!data.texture)
-    g_error ("Failed to load texture");
+    g_error ("Failed to load texture: %s", error->message);
 
   /* a CoglPipeline conceptually describes all the state for vertex
    * processing, fragment processing and blending geometry. When
